from .core import create_engine
from . import ui


def main(server=None, **kwargs):
    engine = create_engine(server)
    engine.server.controller.on_server_reload = ui.initialize
    ui.initialize(engine.server)
    engine.server.start(**kwargs)


if __name__ == "__main__":
    main()



from trame.widgets import vuetify

from . import base_card as bc


def image_card(ctrl):
    with bc.ui_card(
        "image_workflow",
    ):
        bc.ui_card_title(
            title="Image",
            ui_icon="mdi-image-multiple-outline",
        )
        with bc.ui_card_text():
            image_card_text(ctrl)


def image_card_text(ctrl):
    with vuetify.VRow(classes="pa-0 pt-1", dense=True, hide_details=True):
        with vuetify.VCol(cols="6"):
            vuetify.VSelect(
                v_model=("selectedRepresentation",),
                items=("representations",),
                label="Representation",
                dense=True,
                hide_details=True,
                outlined=True,
            )
        with vuetify.VCol(cols="6"):
            vuetify.VSelect(
                v_model=("selectedColor",),
                items=("colors",),
                label="Colors",
                dense=True,
                hide_details=True,
                outlined=True,
            )
    with vuetify.VRow(classes="pa-0 pt-1", dense=True, hide_details=True):
        with vuetify.VCol(cols="6"):
            vuetify.VCheckbox(
                v_model=("contour_labels",),
                label="Contour labels",
                dense=True,
                hide_details=True,
            )



from trame.widgets import html, vuetify

# -----------------------------------------------------------------------------
# Base Card Components
# -----------------------------------------------------------------------------


def ui_card_title(title, ui_icon):
    with vuetify.VCardTitle(
        classes="grey lighten-1 pa-0 grey--text text--darken-3",
        style="user-select: none; cursor: pointer",
        hide_details=True,
        dense=True,
    ) as card_title:
        vuetify.VIcon(ui_icon, classes="mr-3", color="grey darken-3")
        html.Div(title)
    return card_title


def ui_card_text():
    card_text = vuetify.VCardText(classes="pa-1")
    return card_text


def ui_card_actions():
    card_actions = vuetify.VCardActions(
        classes="px-0 py-1",
        hide_details=True,
        dense=True,
    )
    return card_actions


def ui_card(ui_name):
    card = vuetify.VCard(
        classes="ma-1 rounded elevation-8",
        v_show=ui_name,
    )
    return card



from trame.widgets import vuetify

from . import base_card as bc


def device_card(ctrl):
    with bc.ui_card(
        "device_workflow",
    ):
        bc.ui_card_title(
            title="Device",
            ui_icon="mdi-microscope",
        )
        with bc.ui_card_text():
            device_card_text(ctrl)


def device_card_text(ctrl):
    with vuetify.VRow(classes="pa-0 pt-1", dense=True, hide_details=True):
        with vuetify.VCol(cols="6"):
            vuetify.VSelect(
                v_model=("selectedDevice",),
                items=("devices",),
                label="Device",
                dense=True,
                hide_details=True,
                outlined=True,
            )
        with vuetify.VCol(cols="6"):
            vuetify.VSelect(
                v_model=("device_active_data", None),
                items=("device_data", ["", "mask", "efficiency", "error"]),
                label="Parameters",
                dense=True,
                hide_details=True,
                outlined=True,
            )



from trame.widgets import html, vuetify

from . import base_card as bc


def on_click(e):
    print(e)


def directory_card(ctrl):
    with bc.ui_card(
        "directory_workflow",
    ):
        bc.ui_card_title(
            title="Directory",
            ui_icon="mdi-folder-eye-outline",
        )
        with bc.ui_card_text():
            directory_card_text(ctrl)


def directory_card_text(ctrl):
    with vuetify.VRow(classes="pa-0 pt-1", dense=True):
        with vuetify.VCol(cols="1"):
            directory_dialog(ctrl)
        with vuetify.VCol(cols="11"):
            vuetify.VTextField(
                classes="mb-1",
                dense=True,
                hide_details=True,
                readonly=True,
                outlined=True,
                disabled=True,
                type="string",
                v_model=("directory_label",),
            )


def directory_dialog(ctrl):
    with vuetify.VDialog(
        v_model=("directory_dialog", False), persistent=True, max_width="300px"
    ):
        with vuetify.Template(
            v_slot_activator="{ on, attrs }",
        ):
            with vuetify.VBtn(
                small=True,
                color="primary",
                dark=True,
                icon=True,
                v_bind="attrs",
                v_on="on",
            ):
                vuetify.VIcon("mdi-folder")
        with vuetify.VCard(
            classes="ma-1 rounded elevation-8",
        ):
            with vuetify.VCardTitle(
                classes="grey lighten-1 pa-0 grey--text text--darken-3",
                style="user-select: none; cursor: pointer",
                hide_details=True,
                dense=True,
            ):
                html.Div("Directory")
            with vuetify.VCardText(classes="pa-1"):
                vuetify.VTreeview(
                    classes="mb-1",
                    dense=True,
                    hide_details=True,
                    v_model=("dirtree",),
                    items=("dirs",),
                    activatable=True,
                    item_key="path",
                    open_on_click=True,
                    update_active=(
                        ctrl.trigger_directory_selection,
                        "[$event]",
                    ),
                )
            with vuetify.VCardActions(
                classes="px-0 py-1",
                hide_details=True,
                dense=True,
            ):
                vuetify.VSpacer()
                with vuetify.VBtn(
                    classes="ma-0",
                    icon=True,
                    dense=True,
                    small=True,
                    tile=True,
                    color="error",
                    click="directory_dialog = false",
                ):
                    vuetify.VIcon("mdi-cancel")
                with vuetify.VBtn(
                    classes="ma-0",
                    icon=True,
                    dense=True,
                    small=True,
                    tile=True,
                    color="success",
                    click="directory_dialog = false",
                ):
                    vuetify.VIcon("mdi-check")



from trame.widgets import vuetify

from . import base_card as bc


def browse_card(ctrl):
    with bc.ui_card(
        "browse_workflow",
    ):
        bc.ui_card_title(
            title="Browse",
            ui_icon="mdi-application-outline",
        )
        with bc.ui_card_text():
            browse_card_text(ctrl)


def browse_card_text(ctrl):
    with vuetify.VRow(classes="pa-0 pt-1", dense=True):
        with vuetify.VList(
            dense=True,
            hide_details=True,
        ):
            with vuetify.VListItemGroup(
                v_model=("file_selected",),
                color="primary",
            ):
                with vuetify.VListItem(
                    v_for="(item, index) in files",
                    key="index",
                    dense=True,
                    hide_details=True,
                    click=(ctrl.selected_file, "[item]"),
                    color="primary",
                    style="min-height:10px",
                ):
                    with vuetify.VListItemIcon(classes="ma-0 mr-1"):
                        vuetify.VIcon("mdi-file", x_small=True)
                    with vuetify.VListItemContent(classes="pa-0"):
                        vuetify.VListItemTitle("{{item}}")



from trame.widgets import vuetify


def workflow_buttons():
    vuetify.VCheckbox(
        v_model=("device_workflow", False),
        color="success",
        on_icon="mdi-microscope",
        off_icon="mdi-microscope",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    vuetify.VCheckbox(
        v_model=("directory_workflow", False),
        color="success",
        on_icon="mdi-folder-eye",
        off_icon="mdi-folder-eye",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    vuetify.VCheckbox(
        v_model=("browse_workflow", False),
        color="success",
        on_icon="mdi-application",
        off_icon="mdi-application",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    vuetify.VCheckbox(
        v_model=("scattering_workflow", False),
        color="success",
        on_icon="mdi-scatter-plot",
        off_icon="mdi-scatter-plot",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    vuetify.VCheckbox(
        v_model=("transmission_workflow", False),
        color="success",
        on_icon="mdi-transfer",
        off_icon="mdi-transfer",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )
    vuetify.VCheckbox(
        v_model=("image_workflow", False),
        color="success",
        on_icon="mdi-image-multiple",
        off_icon="mdi-image-multiple",
        classes="mx-1",
        hide_details=True,
        dense=True,
    )



from trame.ui.vuetify import SinglePageWithDrawerLayout
from trame.widgets import vuetify, plotly, trame

from . import workflow_buttons as wb
from . import device_card as devc
from . import directory_card as dc
from . import browse_card as bc
from . import scattering_files_card as sfc
from . import transmission_files_card as tfc
from . import image_card as ic


def initialize(server):
    ctrl = server.controller

    with SinglePageWithDrawerLayout(server) as layout:
        # Toolbar
        layout.title.set_text("<strong>AIR-SANS</strong>")
        with layout.icon:
            vuetify.VIcon("mdi-bullseye")
        with layout.toolbar:
            vuetify.VSpacer()
            wb.workflow_buttons()
            vuetify.VSpacer()
        with layout.drawer as drawer:
            drawer.width = 350
            devc.device_card(ctrl)
            dc.directory_card(ctrl)
            bc.browse_card(ctrl)
            sfc.scattering_files_card(ctrl)
            tfc.transmission_files_card(ctrl)
            ic.image_card(ctrl)
        # Main content
        with layout.content:
            with vuetify.VContainer(
                fluid=True,
                classes="pa-0 fill-height",
            ):
                with trame.SizeObserver("d11_size"):
                    ctrl.update_d11 = plotly.Figure(
                        display_mode_bar=("false",),
                        v_show=("figure_ready", False),
                    ).update

        # Footer
        # layout.footer.hide()



r"""
Define your classes and create the instances that you need to expose
"""
import os

import numpy

import logging
from trame.app import get_server
from trame.decorators import TrameApp, change

from .instrument.d11_plus import D11_Plus

from .visualization import create_visualization

from .utilities import file_search as fs
from .utilities import file_load as fl

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


SUPPORTED_DEVICES = ["D11+"]

# ---------------------------------------------------------
# Engine class
# ---------------------------------------------------------


@TrameApp()
class Engine:
    def __init__(self, server=None):
        self.server = get_server(server, client_type="vue2")
        self.server.cli.add_argument(
            "--data", help="Path to browse", dest="data", default="."
        )
        args, _ = server.cli.parse_known_args()

        # initialize state + controller
        state, ctrl = server.state, server.controller

        Visualization = create_visualization(server)
        self.Visualization = Visualization

        # Local variables
        self._selected_device = None

        # search directory contents
        dirs = fs.get_directory_structure(args.data)
        state.devices = SUPPORTED_DEVICES
        state.selectedDevice = None
        state.directory = None
        state.directory_label = None
        state.dirs = dirs
        state.dirtree = []
        state.directory_dialog = False
        state.files = []
        state.file_selected = None
        state.file = None

        # Initialize data
        state.data = None
        state.center_data = None
        state.scattering_samples_28m = []
        state.scattering_samples_8m = []
        state.scattering_samples_2m = []
        state.scattering_empty_cell_28m = []
        state.scattering_empty_cell_8m = []
        state.scattering_empty_cell_2m = []
        state.scattering_blocked_beam_28m = []
        state.scattering_blocked_beam_8m = []
        state.scattering_blocked_beam_2m = []
        state.transmission_samples_28m = []
        state.transmission_samples_8m = []
        state.transmission_samples_2m = []
        state.transmission_empty_cell_28m = []
        state.transmission_empty_cell_8m = []
        state.transmission_empty_cell_2m = []
        state.transmission_direct_beam_28m = []
        state.transmission_direct_beam_8m = []
        state.transmission_direct_beam_2m = []

        # Set workflows state
        state.directory_workflow = False
        state.browse_workflow = False
        state.scattering_workflow = False
        state.transmission_workflow = False
        state.image_workflow = False

        # Set state visualization
        state.pixel_ratio = 2.0
        state.representations = ["Heatmap", "Contours", "Combined"]
        state.selectedRepresentation = "Heatmap"
        state.colors = ["spectral", "rdbu", "gray", "blackbody", "sunset"]
        state.selectedColor = "spectral"
        state.contour_labels = True

        # Set state variable
        state.trame__title = "air-sans"

        # Bind instance methods to controller
        ctrl.trigger_directory_dialog = self.trigger_directory_dialog
        ctrl.trigger_directory_selection = self.trigger_directory_selection
        ctrl.selected_file = self.selected_file

        @state.change("d11_size")
        def update_left_contour_size(d11_size, **kwargs):
            if d11_size is None:
                return

            Visualization.create_d11_fig(**d11_size.get("size"))

    # @state.change("left_contour_size")
    # def update_left_contour_size(left_contour_size, **kwargs):
    #   if left_contour_size is None:
    #       return
    #
    #   ctrl.update_left_contour(Visualization.create_left_contour_fig(**left_contour_size.get("size")))

    # @state.change("center_contour_size")
    # def update_center_contour_size(center_contour_size, **kwargs):
    #    if center_contour_size is None:
    #        return

    #    ctrl.update_center_contour(Visualization.create_center_contour_fig(**center_contour_size.get("size")))

    # @state.change("right_contour_size")
    # def update_right_contour_size(right_contour_size, **kwargs):
    #    if right_contour_size is None:
    #        return
    #
    #    ctrl.update_right_contour(Visualization.create_right_contour_fig(**right_contour_size.get("size")))

    @property
    def state(self):
        return self.server.state

    @property
    def ctrl(self):
        return self.server.controller

    @change("selectedDevice")
    def on_device_change(self, selectedDevice, **kwargs):
        if selectedDevice == "D11+":
            self._selected_device = D11_Plus()

    def trigger_directory_dialog(self):
        self.server.state.directory_dialog = not self.server.state.directory_dialog

    def trigger_directory_selection(self, active_nodes):
        self.server.state.directory_label = None
        self.server.state.directory = None
        if len(active_nodes):
            node_id = active_nodes[0]
            if isinstance(node_id, str):
                self.server.state.directory_label = os.path.basename(node_id)
                self.server.state.directory = node_id
                self.server.state.files = fs.get_file_list(node_id)

    def selected_file(self, file):
        state = self.server.state
        state.file = file
        data = fl.load(state.directory, state.file)
        pixel_y = 1.0
        pixel_x = 1.0
        print(pixel_y / pixel_x, pixel_x, type(pixel_x), pixel_y, type(pixel_y))
        pixel_y = data["/entry0/D11/Detector 1/pixel_size_y"].nxdata[0].item()
        pixel_x = data["/entry0/D11/Detector 1/pixel_size_x"].nxdata[0].item()
        print(pixel_y / pixel_x, pixel_x, type(pixel_x), pixel_y, type(pixel_y))
        state.pixel_ratio = pixel_y / pixel_x
        print(state.pixel_ratio)

        detector1 = data["/entry0/D11/Detector 1/data"]
        detector1_2d = detector1[:, :, 0]
        ny, nx = detector1_2d.shape
        state.center_ny = ny
        state.center_nx = nx
        d1 = numpy.zeros(detector1_2d.shape, dtype=numpy.int32, order="C")
        for i in range(0, nx - 1):
            for j in range(0, ny - 1):
                d1[j, i] = detector1_2d[j, i]
        self.Visualization.set_center_data(d1.transpose())

        detector2 = data["/entry0/D11/Detector 2/data"]
        detector2_2d = detector2[:, :, 0]
        ny, nx = detector2_2d.shape
        state.left_ny = ny
        state.left_nx = nx
        d2 = numpy.zeros(detector2_2d.shape, dtype=numpy.int32, order="C")
        for i in range(0, nx - 1):
            for j in range(0, ny - 1):
                d2[j, i] = detector2_2d[j, i]
        self.Visualization.set_left_data(d2.transpose())

        detector3 = data["/entry0/D11/Detector 3/data"]
        detector3_2d = detector3[:, :, 0]
        ny, nx = detector3_2d.shape
        state.right_ny = ny
        state.right_nx = nx
        d3 = numpy.zeros(detector3_2d.shape, dtype=numpy.int32, order="C")
        for i in range(0, nx - 1):
            for j in range(0, ny - 1):
                d3[j, i] = detector3_2d[j, i]
        self.Visualization.set_right_data(d3.transpose())

        self.Visualization.create_d11_fig()
        state.figure_ready = True

    @change("device_active_data")
    def show_imask(self, device_active_data, **kwargs):
        self.server.state.figure_ready = False

        if device_active_data == "mask":
            self.Visualization.set_center_data(
                self._selected_device.detector1_imask_data
            )
            self.Visualization.set_left_data(self._selected_device.detector2_imask_data)
            self.Visualization.set_right_data(
                self._selected_device.detector3_imask_data
            )
            self.server.state.figure_ready = True
            self.Visualization.create_d11_fig()

        if device_active_data == "efficiency":
            self.Visualization.set_center_data(
                self._selected_device.detector1_efficiency_data
            )
            self.Visualization.set_left_data(
                self._selected_device.detector2_efficiency_data
            )
            self.Visualization.set_right_data(
                self._selected_device.detector3_efficiency_data
            )
            self.server.state.figure_ready = True
            self.Visualization.create_d11_fig()

        if device_active_data == "error":
            self.Visualization.set_center_data(
                self._selected_device.detector1_efficiency_error
            )
            self.Visualization.set_left_data(
                self._selected_device.detector2_efficiency_error
            )
            self.Visualization.set_right_data(
                self._selected_device.detector3_efficiency_error
            )
            self.server.state.figure_ready = True
            self.Visualization.create_d11_fig()

        if device_active_data == "" and self.server.state.file:
            self.selected_file(self.server.state.file)


def create_engine(server=None):
    # Get or create server
    if server is None:
        server = get_server(client_type="vue2")

    if isinstance(server, str):
        server = get_server(server, client_type="vue2")

    return Engine(server)



import numpy as np

from pathlib import Path
from . import mat_files as mf

BASE = Path(__file__).parent


class D11_Plus:
    def __init__(self):
        self.Facility = "ILL"
        self.Instrument = "d11_plus"
        self.Name = "D11+_(2021-current)"
        self.Highlight = True
        # Filename
        self.filename_numeric_length = 6
        self.filename_lead_string = []
        self.filename_tail_string = []
        self.filename_extension_string = [".nxs"]
        self.filename_data_loader = "raw_read_ill_nexus_d11d22_2021"
        # Misc
        self.guide_size = [45e-3, 45e-3]  # [x,y]
        self.attenuators = [1, 112.8, 421.14, 1911.96]  # Attenautors 0, 1, 2, 3 etc.
        # Detector(s)
        self.detectors = 3
        # Main Panel
        self.detector1_type = "tube"
        self.detector1_view_position = "centre"
        self.detector1_name = "Main Panel"
        self.detector1_det_offset = (
            0  # position difference between panel and main detector
        )
        self.detector1_pixels = [256, 192]  # Number of pixels, x,y
        self.detector1_pixel_size = [4, 8]
        # mm [x y]
        self.detector1_nominal_beam_centre = [128.5, 103.5]  # [x y]
        self.detector1_nominal_det_translation = [0, 0]  # mm [x y]
        self.detector1_dead_time = 1.5e-6 * np.ones(
            (192, 1)
        )  # the direction of this matrix determines the direction of the tubes
        self.detector1_imask_data = mf.read_mask_mat_file(
            BASE / "det_mask" / "det1_d11_plus.mat"
        )
        self.detector1_relative_efficiency = (
            1  # Efficiency relative to rear detector (detector 1)
        )
        self.detector1_efficiency_data = mf.read_efficiency_mat_file_data(
            BASE / "det_efficiency" / "det1_d11_plus.mat"
        )
        self.detector1_efficiency_error = mf.read_efficiency_mat_file_error(
            BASE / "det_efficiency" / "det1_d11_plus.mat"
        )
        # Left Panel
        self.detector2_type = "tube"
        self.detector2_view_position = "left"
        self.detector2_name = "Left"
        self.detector2_det_offset = (
            -0.105
        )  # position difference between panel and main detector
        self.detector2_pixels = [32, 256]  # Number of pixels, x,y
        self.detector2_pixel_size = [8, 4]  # mm [x y]
        self.detector2_nominal_beam_centre = [16.5, 142.5]  # [x y]
        self.detector2_nominal_det_translation = [-628, 0]  # mm [x y]
        self.detector2_dead_time = 1.5e-6 * np.ones((1, 32))
        self.detector2_imask_data = mf.read_mask_mat_file(
            BASE / "det_mask" / "det2_d11_plus.mat"
        )
        self.detector2_relative_efficiency = 1
        self.detector2_efficiency_data = mf.read_efficiency_mat_file_data(
            BASE / "det_efficiency" / "det2_d11_plus.mat"
        )
        self.detector2_efficiency_error = mf.read_efficiency_mat_file_error(
            BASE / "det_efficiency" / "det2_d11_plus.mat"
        )
        # Right Panel
        self.detector3_type = "tube"
        self.detector3_view_position = "right"
        self.detector3_name = "Right"
        self.detector3_det_offset = (
            -0.105
        )  # position difference between panel and main detector
        self.detector3_pixels = [32, 256]  # Number of pixels, x,y
        self.detector3_pixel_size = [8, 4]  # mm [x y]
        self.detector3_nominal_beam_centre = [16.5, 142.5]  # [x y]
        self.detector3_nominal_det_translation = [+628, 0]  # mm [x y]
        self.detector3_dead_time = 1.5e-6 * np.ones((1, 32))
        self.detector3_imask_data = mf.read_mask_mat_file(
            BASE / "det_mask" / "det3_d11_plus.mat"
        )
        self.detector3_relative_efficiency = 1
        self.detector3_efficiency_data = mf.read_efficiency_mat_file_data(
            BASE / "det_efficiency" / "det3_d11_plus.mat"
        )
        self.detector3_efficiency_error = mf.read_efficiency_mat_file_error(
            BASE / "det_efficiency" / "det3_d11_plus.mat"
        )
